@@ -6,8 +6,8 @@ use CGI::ContactForm;
 
 contactform (
     recname   => 'John Smith',
-    recmail   => 'john.smith@domain.com',
+    recmail   => 'john.smith@example.com',
     styleurl  => '/style/ContactForm.css',
 );
 
-# $Id: contact.pl,v 1.5 2004/03/15 01:09:12 Gunnar Hjalmarsson Exp $
+# $Id: contact.pl,v 1.6 2004/08/10 23:25:21 gunnarh Exp $
